package trains.persistence.binary;

import trains.model.*;
import trains.persistence.*;
import java.util.*;
import java.io.*;

public class CityStations implements CityStationsReader
{
	private ObjectInputStream innerStream;
	private Map<String, SortedSet<Station>> map;
	private Map<String, City> stationCityMap;

	public CityStations(InputStream input) throws IOException, InvalidDataException
	{
		this.innerStream = new ObjectInputStream(input);
		map = new TreeMap<String, SortedSet<Station>>();
		stationCityMap = new HashMap<String, City>();
		this.createMap();
		innerStream.close();
	}

	@Override
	public Map<String, SortedSet<Station>> getMap()
	{
		return map;
	}

	@Override
	public SortedSet<Station> getStations(City city)
	{
		return getStations(city.getName());
	}

	@Override
	public SortedSet<Station> getStations(String city)
	{
		return getMap().get(city);
	}

	@Override
	public City getCity(String station)
	{
		station = station.toUpperCase().trim();
		return stationCityMap.containsKey(station) ? 
				stationCityMap.get(station) : 
					null;
		
//      Alternativa senza l'uso della mappa stationCityMap
//		StringTokenizer tok = new StringTokenizer(station.toUpperCase(), " ");
//
//		String name = tok.nextToken();
//
//		City c = null;
//
//		while (c == null)
//		{
//			if (map.containsKey(name))
//			{
//				c = new City(name, map.get(name));
//			}
//			else
//			{
//				try
//				{
//					name = name + " " + tok.nextToken();
//				}
//				catch (NoSuchElementException e)
//				{
//					break;
//				}
//			}
//		}
//
//		return c;
	}

	private void createMap() throws IOException, InvalidDataException
	{
		try
		{
			int number = innerStream.readInt();

			for (int i = 0; i < number; i++)
			{
				City city = (City) innerStream.readObject();
				map.put(city.getName(), city.getStations());
				for(Station station : city.getStations())
				{
					String fullStationName = station.toString().trim().toUpperCase();
					stationCityMap.put(fullStationName, city);
				}
			}
		}
		catch (ClassNotFoundException e) // Problemi deserializzazione
		{
			throw new InvalidDataException("Dati non validi", e);
		}
		catch (InvalidClassException e)
		{
			throw new InvalidDataException("Dati non validi", e);
		}
		catch (EOFException e) // File terminato
		{
		}

	}

//  In alternativa al trim su station.toString()
//	private String getFullStationName(Station station)
//	{
//		String fullStationName = station.getCityName();
//		if (station.getStationName() != null && !station.getStationName().isEmpty())
//			fullStationName += " " + station.getStationName();
//		return fullStationName;
//	}

}
