package trains.tests;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.util.Comparator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import trains.model.City;
import trains.model.Station;
import trains.persistence.InvalidDataException;
import trains.persistence.binary.CityStations;

public class CityStationsMock extends CityStations {
	private static InputStream getEmptyData() throws IOException
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(bos);
		oos.writeInt(0);
		oos.flush();
		oos.close();
		bos.close();
		return new ByteArrayInputStream(bos.toByteArray());
	}

	public CityStationsMock() throws IOException, InvalidDataException
	{
		super (getEmptyData());
	}

	public City getCity(String station)
	{
		if ("BOLOGNA CENTRALE".equals(station.toUpperCase()))
		{
			return new City("BOLOGNA");
		}
		if ("CASTEL S.PIETRO".equals(station.toUpperCase()))
		{
			return new City("CASTEL S.PIETRO");
		}
		if ("RIMINI".equals(station.toUpperCase()))
		{
			return new City("RIMINI");
		}
		
		if ("RIMINI MIRAMARE".equals(station.toUpperCase()))
		{
			return new City("RIMINI");
		}
		return null;
	}

	
	@Override
	public Map<String, SortedSet<Station>>	getMap()
	{
		TreeMap<String, SortedSet<Station>> mappa= new TreeMap<String, SortedSet<Station>>(
				new Comparator<String>()
				{
					@Override
					public int compare(String s1, String s2)
					{
						return s1.compareToIgnoreCase(s2);
					}
				}
		);
		TreeSet<Station> s= new TreeSet<Station>();
		s.add(new Station("Bologna", "Centrale"));
		s.add(new Station("Bologna", "Corticella"));
		mappa.put("Bologna", s);
		s= new TreeSet<Station>();
		s.add(new Station("Castel S.Pietro", ""));
		mappa.put("Castel S.Pietro", s);
		
		s= new TreeSet<Station>();
		s.add(new Station("Rimini", ""));
		s.add(new Station("Rimini", "Miramare"));
		mappa.put("Rimini", s);
		
		return mappa;	
		
	}
	
}
