package trains.ui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import trains.model.*;
import trains.persistence.binary.CityStationsReader;

public class TrainFrame extends JFrame implements ActionListener
{

	private static final long serialVersionUID = 1L;

	private Controller controller;

	private JComboBox departureCityComboBox;
	private JComboBox arrivalCityComboBox;
	private TrainDateSpinner departureDateSpinner;
	private JTextArea trainListArea;
	private CityStationsReader cities;
	private StringBuilder trainSchedule;
	private JButton printButton;
	private JButton searchButton;

	public TrainFrame(Controller controller, CityStationsReader cities)
	{
		this.controller = controller;
		this.cities = cities;
		trainSchedule = new StringBuilder();
		initGUI();
		bindData();
	}

	private void bindData()
	{
		for (String city : controller.getSortedCities())
		{
			departureCityComboBox.addItem(city);
			arrivalCityComboBox.addItem(city);
		}
	}

	private void initGUI()
	{
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout(5, 5));
		setSize(640, 480);

		// Controlli selezione
		JPanel leftPanel = new JPanel();
		{
			leftPanel.setLayout(new BorderLayout());
			JPanel upperLeftPanel = new JPanel();
			{
				upperLeftPanel.setLayout(new BoxLayout(upperLeftPanel, BoxLayout.PAGE_AXIS));

				JLabel label = new JLabel("Departure City:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				departureCityComboBox = new JComboBox();
				upperLeftPanel.add(departureCityComboBox);

				label = new JLabel("Arrival City:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				arrivalCityComboBox = new JComboBox();
				upperLeftPanel.add(arrivalCityComboBox);

				label = new JLabel("Departure Date:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				departureDateSpinner = new TrainDateSpinner();

				upperLeftPanel.add(departureDateSpinner);

				searchButton = new JButton("Find");
				searchButton.setAlignmentX(CENTER_ALIGNMENT);
				searchButton.addActionListener(this);
				upperLeftPanel.add(searchButton);
			}
			leftPanel.add(upperLeftPanel, BorderLayout.PAGE_START);
		}
		getContentPane().add(leftPanel, BorderLayout.LINE_START);

		JPanel rightPanel = new JPanel();
		{
			// rightPanel.setLayout(new BorderLayout(5, 5));
			rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
			rightPanel.add(new JLabel("Travel Solutions:"), BorderLayout.CENTER);
			trainListArea = new JTextArea();
			trainListArea.setAlignmentX(CENTER_ALIGNMENT);
			rightPanel.add(trainListArea, BorderLayout.CENTER);
			printButton = new JButton("Print");
			printButton.addActionListener(this);
			printButton.setAlignmentX(CENTER_ALIGNMENT);
			rightPanel.add(printButton, BorderLayout.CENTER);
		}
		getContentPane().add(rightPanel, BorderLayout.CENTER);
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		Object button = e.getSource();
		if (button == searchButton)
		{
			showResults();
		}
		if (button == printButton)
		{
			printResults();
		}

	}

	private void showResults()
	{

		String departureCity = (String) departureCityComboBox.getSelectedItem();
		String arrivalCity = (String) arrivalCityComboBox.getSelectedItem();
		Date date = departureDateSpinner.getDateValue();

		Set<Station> departureStations = cities.getStations(departureCity);
		Set<Station> arrivalStations = cities.getStations(arrivalCity);
		trainSchedule = new StringBuilder();
		trainListArea.setText("");
		for (Station dep : departureStations)
		{
			for (Station arr : arrivalStations)
			{
				Map<Train, Time> mappa;

				mappa = controller.searchTrains(dep, arr, date);
				// trainListArea.append("Treni trovati tra: " + dep.toString()
				// +" "+ arr.toString()+" "+mappa.size()+"\n");
				if (mappa.size() > 0)
				{
					Set<Train> trains = mappa.keySet();
					for (Train t : trains)
					{
						Time time = mappa.get(t);
						String result = "Treno: " + t.toString() + " da " + dep.toString() + " a " + arr.toString()
								+ " Tempo di percorrenza: " + time.toString() + "\n";
						trainListArea.append(result);
						trainSchedule.append(result);
					}
				}
			}
		}
		trainListArea.append("Non ci sono altre soluzioni\n");
		trainSchedule.append("Non ci sono altre soluzioni\n");

	}

	private void printResults()
	{
		FileOutputStream file;
		try
		{
			file = new FileOutputStream("Results.txt");
			PrintWriter printw = new PrintWriter(file);
			printw.print(trainSchedule);
			printw.close();
			file.close();
		}
		catch (FileNotFoundException e1)
		{
			e1.printStackTrace();
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}

	}

}
