package trains.model;

public class Stop implements Comparable<Stop>
{
	private Station station = null;
	private Time arrival = null;
	private Time departure = null;

	public Stop(Station station, Time arrival, Time departure)
	{
		if (station == null)
			throw new IllegalArgumentException("Null Station");
		else
			this.station = station;
		if (arrival == null && departure == null)
			throw new IllegalArgumentException("Arrival and Departure cannot be both null");
		else
			this.station = station;

		if (arrival != null && arrival.isValid())
			this.arrival = arrival;
		if (departure != null && departure.isValid())
			this.departure = departure;
	}

	public Station getStation()
	{
		return station;
	}

	public void setStation(Station station)
	{
		this.station = station;
	}

	public Time getArrival()
	{
		return arrival;
	}

	public void setArrival(Time arrival)
	{
		this.arrival = arrival;
	}

	public Time getDeparture()
	{
		return departure;
	}

	public void setDeparture(Time departure)
	{
		this.departure = departure;
	}

	@Override
	public String toString()
	{
		return station.getStationName() + ", " + (arrival == null ? "--:--" : arrival.toString()) + ", "
				+ (departure == null ? "--:--" : departure.toString()) + System.getProperty("line.separator");

	}

	@Override
	public int compareTo(Stop o)
	{
		if (arrival == null)
		{
			return -1;
		}
		if (o.arrival == null)
		{
			return 1;
		}
		return arrival.compareTo(o.arrival);
	}

	@Override
	public boolean equals(Object o)
	{
		if (!(o instanceof Stop))
		{
			return false;
		}

		Stop st = (Stop) o;

		if (st.getArrival() == null)
		{
			if (arrival != null)
			{
				return false;
			}
		}
		else
		{
			if (!st.getArrival().equals(arrival))
			{
				return false;
			}
		}

		if (st.getDeparture() == null)
		{
			if (departure != null)
			{
				return false;
			}
		}
		else
		{
			if (!st.getDeparture().equals(departure))
			{
				return false;
			}
		}

		if (st.getStation() == null)
		{
			if (station != null)
			{
				return false;
			}
		}
		else
		{
			if (!st.getStation().equals(station))
			{
				return false;
			}
		}

		return true;
	}
}
